"""
Authors: Lyn Turbak
Consulted:
Date: 2022-03-29
Purpose: Test file for wordSearch
"""

import optimism
import wordSearch

CS_TEST_PUZZLE = wordSearch.TEST_PUZZLE
CS_TEST_WORDS = wordSearch.TEST_WORDS

TEA_PUZZLE = [
    'bdtiwtoolongfrpsooxp',
    'bfrzjkfjpwssteeprcmm',
    'kkkijasmineyrtbyohyn',
    'emtygaiqeqkatucboamf',
    'tceisvcukyguwvdiimyr',
    'tsasafyzaylwtatcbojz',
    'lvpfufzearlgreyhomah',
    'eiopctzxdrkrldsasifu',
    'odtwegkldzuphggiclxa',
    'xtlvrmftmgmyxtxpheal',
    ]
TEA_WORDS = [
   'chai',
   'oolong',
   'earlgrey',
   'rooibos',
   'saucer',
   'teapot',
   'chamomile',
   'jasmine',
   'kettle',
   'steep',
   ]

testPG = optimism.testFunction(wordSearch.printGrid)
testPG.case(['abcd', 'efgh', 'ijkl'])\
      .checkPrintedLines(
          "a b c d ",
          "",
          "e f g h ",
          "",
          "i j k l ",
          ""
      )
testPG.case(['ab', 'cd', 'ef', 'gh', 'ij'])\
      .checkPrintedLines(
          "a b ",
          "",
          "c d ",
          "",
          "e f ",
          "",
          "g h ",
          "",
          "i j ",
          ""
      )
testPG.case(CS_TEST_PUZZLE)\
      .checkPrintedLines(
          "- - - - s - - - - - - - ",
          "",
          "- - - - c e n t e r - - ",
          "",
          "p - - - i - - r - - - - ",
          "",
          "y e l s e l l e w - - - ",
          "",
          "t - o - n - - t - - - - ",
          "",
          "h - o - c - - u - - - 1 ",
          "",
          "o - p - e - - p - - 1 - ",
          "",
          "n e s t e d - m - 1 - - ",
          "",
          "- - - - - - - o s - - - ",
          "",
          "- - - - - - - c - - - - ",
          ""
      )

testGC = optimism.testFunction(wordSearch.getColumns)
testGC.case([ "abcd", "efgh", "ijkl" ])\
      .checkReturnValue(['aei', 'bfj', 'cgk', 'dhl'])
testGC.case(['ab', 'cd', 'ef', 'gh', 'ij'])\
      .checkReturnValue(['acegi', 'bdfhj'])
testGC.case(CS_TEST_PUZZLE)\
      .checkReturnValue([
        '--python--',
        '---e---e--',
        '---loops--',
        '---s---t--',
        'sciencee--',
        '-e-l---d--',
        '-n-l------',
        '-tretupmoc',
        '-e-w----s-',
        '-r-----1--',
        '------1---',
        '-----1----',
        ])

testFH = optimism.testFunction(wordSearch.findHorizontal)
testFH.case(['----', 'find', '-me-'], 'find')\
      .checkReturnValue([(1, 0)])
testFH.case(['----', 'find', '-me-'], 'me')\
      .checkReturnValue([(2, 1)])
testFH.case(['dnif', '-e--', '-m--'], 'find')\
      .checkReturnValue([(0, 0)])
testFH.case(['dnif', '-e--', '-m--'], 'me')\
      .checkReturnValue([])
testFH.case(CS_TEST_PUZZLE, 'center')\
       .checkReturnValue([(1, 4)])
testFH.case(CS_TEST_PUZZLE, 'nested')\
       .checkReturnValue([(7, 0)])
testFH.case(TEA_PUZZLE, 'oolong').checkReturnValue([(0, 6)])
testFH.case(TEA_PUZZLE, 'earlgrey').checkReturnValue([(6, 7)])
testFH.case(TEA_PUZZLE, 'jasmine').checkReturnValue([(2, 4)])
testFH.case(TEA_PUZZLE, 'steep').checkReturnValue([(1, 11)])


testFW = optimism.testFunction(wordSearch.findWords)
testFW.case(['----', 'find', '-me-'], ['find', 'me'])\
       .checkReturnValue([('find', 1, 0), ('me', 2, 1)])
testFW.case(['-f-', '-im', '-ne', '-d-'], ['find', 'me'])\
       .checkReturnValue([('find', 0, 1), ('me', 1, 2)])
testFW.case(['f--', 'i--', 'nme', 'd--'], ['find', 'me'])\
       .checkReturnValue([('find', 0, 0), ('me', 2, 1)])
testFW.case(CS_TEST_PUZZLE, CS_TEST_WORDS)\
       .checkReturnValue([
        ('science', 0, 4),
        ('center', 1, 4),
        ('wellesley', 3, 0),
        ('python', 2, 0),
        ('computer', 2, 7),
        ('nested', 7, 0),
        ('loops', 3, 2),
        ])
testFW.case(TEA_PUZZLE, TEA_WORDS)\
       .checkReturnValue([
        ('chai', 5, 15),
        ('oolong', 0, 6),
        ('earlgrey', 6, 7),
        ('rooibos', 1, 16),
        ('saucer', 4, 4),
        ('teapot', 3, 2),
        ('chamomile', 1, 17),
        ('jasmine', 2, 4),
        ('kettle', 2, 0),
        ('steep', 1, 11),
        ])
