"""
 This is a modified version of the NLTK Word Finder, credited below:

Natural Language Toolkit: Word Finder
Copyright (C) 2001-2021 NLTK Project
Author: Steven Bird <stevenbird1@gmail.com>
URL: <http://nltk.org/>

Simplified from PHP version by Robert Klein <brathna@gmail.com>
http://fswordfinder.sourceforge.net/

Modifications for CS111:
* (Fall 2021) Carolyn Anderson added tea-based default wordlist
  and interface function generatePuzzle for lower-case puzzles. 
* (Spring 2022) Lyn Turbak
"""

import random


def revword(word):
    """reverse a word with probability 0.5"""
    if random.randint(1, 2) == 1:
        return word[::-1]
    return word

# [2022/03/29, lyn]
dirDict = {
    1 : 'diag1',
    2: 'vert',
    3: 'diag2',
    4: 'horiz',
}


def step(word, x, xf, y, yf, grid):
    """try to insert word at position x,y; direction encoded in xf,yf"""
    for i in range(len(word)):
        if grid[xf(i)][yf(i)] != "" and grid[xf(i)][yf(i)] != word[i]:
            return False
    for i in range(len(word)):
        grid[xf(i)][yf(i)] = word[i]
    return True


def check(word, dir, x, y, grid, rows, cols):
    """ try to insert word at position x,y, in direction dir """
    if dir == 1:
        if x - len(word) < 0 or y - len(word) < 0:
            return False
        return step(word, x, lambda i: x - i, y, lambda i: y - i, grid)
    elif dir == 2:
        if x - len(word) < 0:
            return False
        return step(word, x, lambda i: x - i, y, lambda i: y, grid)
    elif dir == 3:
        if x - len(word) < 0 or y + (len(word) - 1) >= cols:
            return False
        return step(word, x, lambda i: x - i, y, lambda i: y + i, grid)
    elif dir == 4:
        if y - len(word) < 0:
            return False
        return step(word, x, lambda i: x, y, lambda i: y - i, grid)


def wordFinder(
    words,
    rows=20,
    cols=20,
    onlyForwardOrDownward=False, 
    attempts=50,
    alph="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
):
    """
    Attempt to arrange words into a letter-grid with the specified
    number of rows and columns.  Try each word in several positions
    and directions, until it can be fitted into the grid, or the
    maximum number of allowable attempts is exceeded.  Returns a tuple
    consisting of the grid and the words that were successfully
    placed.
    :param words: the list of words to be put into the grid
    :type words: list
    :param rows: the number of rows in the grid
    :type rows: int
    :param cols: the number of columns in the grid
    :type cols: int
    :param attempts: the number of times to attempt placing a word
    :type attempts: int
    :param alph: the alphabet, to be used for filling blank cells
    :type alph: list
    :rtype: tuple
    """
    print(f"***wordFinder: onlyForwardOrDownward={onlyForwardOrDownward}")

    # place longer words first
    words = sorted(words, key=len, reverse=True)

    grid = []  # the letter grid
    used = []  # the words we used

    # initialize the grid
    for i in range(rows):
        grid.append([""] * cols)

    # try to place each word
    for word in words:
        word = word.strip().upper()  # normalize
        save = word  # keep a record of the word
        word = (word[::-1] if onlyForwardOrDownward 
                else revword(word))
               # [2022/03/29,lyn] The way algorithm works, must 
               # *reverse* word to go forward or downward!
        print(f"***wordFinder: word={word}")
        for attempt in range(attempts):
            r = random.randint(0, len(word))
            dir = (random.choice([2, 4]) if onlyForwardOrDownward 
                   else random.choice([1, 2, 3, 4]))
                 # [2022/03/29,lyn] 2,4 are horiz & vert; 
                 # 1,3 are diagonal
            print(f"***dir: {dir} ({dirDict[dir]})")
            x = random.randint(0, rows)
            y = random.randint(0, cols)
            if dir == 1:
                x += r
                y += r
            elif dir == 2:
                x += r
            elif dir == 3:
                x += r
                y -= r
            elif dir == 4:
                y += r
            if 0 <= x < rows and 0 <= y < cols:
                if check(word, dir, x, y, grid, rows, cols):
                    #                   used.append((save, dir, x, y, word))
                    used.append(save)
                    break

    # Fill up the remaining spaces
    for i in range(rows):
        for j in range(cols):
            if grid[i][j] == "":
                grid[i][j] = random.choice(alph)

    return grid, used


def randomWordFinder():
    """
    Generates a random puzzle using words from NLTK's words list. Prints
    the puzzle and the list of words used.
    """
    from nltk.corpus import words

    wordlist = words.words()
    random.shuffle(wordlist)
    wordlist = wordlist[:200]
    wordlist = [w for w in wordlist if 3 <= len(w) <= 12]
    grid, used = wordFinder(wordlist)

    print("Word Finder\n")
    for i in range(len(grid)):
        for j in range(len(grid[i])):
            print(grid[i][j], end=" ")
        print()
    print()

    for i in range(len(used)):
        print("{}: {}".format(i + 1, used[i]))


# Carolyn additions:
wordlist = (
    "OOLONG",
    "TEAPOT",
    "KETTLE",
    "EARLGREY",
    "ROOIBOS",
    "CHAMOMILE",
    "STEEP",
    "SAUCER",
    "JASMINE",
    "CHAI"
)


def generatePuzzle(numRows=10, 
                   numCols=20,
                   words=wordlist, 
                   onlyForwardOrDownward=True):
    """
    Generates a puzzle using words from the given wordlist (default is
    a list of 10 tea-related words). The `onlyForwardOrDownward` parameter
    controls whether words can appear backward, upwards, or diagonally;
    `True` means only forward or downward; `False` means all directions.

    Returns a tuple containing

    (1) the grid of letters (a list of `numRows` strings with length
        `numCols`;

    (2) a list of the subset of `words` successfully hidden in the puzzle;

    (3) a list of the subset of `words` unable to be placed in the puzzle. 

    In some cases it's possible that one or more words from the word
    list cannot be fit into the puzzle, so the word list in component #2 will
    not always contain every word in the provided word list; the leftovers
    are in component #3. 

    All letters in the returned grid (which is a list of strings) and
    all letters in each of the returned word lists will be in lower case.
    """
    grid, searchList = wordFinder(words, rows=numRows, cols=numCols, 
                                  onlyForwardOrDownward=True, 
                                  attempts=500)
    lowerGrid = ["".join(line).lower() for line in grid]
    lowerWordSet = {w.lower() for w in words}
    placedWordSet = {w.lower() for w in searchList}
    unplacedWordSet = lowerWordSet.difference(placedWordSet)
    
    return(lowerGrid, # lowercase version of word grid
           list(placedWordSet), # lowercase list of words placed in grid
           list(unplacedWordSet), # lowercase list of words unable to be placed in grid
           )

if __name__ == "__main__":
    # randomWordFinder()
    grid, succeeds, fails = generatePuzzle()
    print(f"Hidden words: {succeeds}\nUnplaced words: {fails}\n"
          f"Puzzle list:\n{grid}\nPuzzle grid:")
    for row in grid: print(row)
    

